import os
import requests
from dotenv import load_dotenv
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
import urllib3
from urllib3.exceptions import InsecureRequestWarning
urllib3.disable_warnings(category=InsecureRequestWarning)

load_dotenv()
tenant_id = os.getenv("tenant_id")
client_id = os.getenv("client_id")
client_secret = os.getenv("client_secret")
resource = os.getenv("resource")

def get_access_token():
    session = requests.Session()
    retries = Retry(total=5,             
                    backoff_factor=1,     
                    status_forcelist=[500, 502, 503, 504],  
                    allowed_methods=["POST"])    
    adapter = HTTPAdapter(max_retries=retries)
    session.mount('https://', adapter)
    session.mount('http://', adapter)
    url = f'https://login.microsoftonline.com:443/{tenant_id}/oauth2/token'
    print(f"URL: {url}")
    data = {
        'client_id': client_id,
        'client_secret': client_secret,
        'resource': resource,
        'grant_type': 'client_credentials',
    }
    response = session.post(url, data=data, verify=False)
    access_token = response.json().get('access_token')

    return access_token





import requests
from dotenv import load_dotenv
from .access_token import get_access_token
load_dotenv()

def get_data(input):

    access_token = get_access_token()
    entity_name = input.lstrip('\n')
    print(f"here is entity {entity_name}")
    url2 = f"https://orgd7246cd1.api.crm4.dynamics.com/api/data/v9.2/{entity_name}"

    headers = {
        'Authorization': f'Bearer {access_token}',
        'Content-Type': 'application/json',
        'OData-MaxVersion': '4.0',
        'OData-Version': '4.0',
    }

    response = requests.get(url2, headers=headers)

    if response.status_code == 200:
        record = response.json()
        if record["value"] == [''] or not record["value"]:
            return False
        return record
    else:
        print(f"Error: {response.status_code}")
    
   



def format_data(json_response):
    formatted_data = []
    for data in json_response.get('value', []):
        data_details = []
        for key, value in data.items():
            if not key.startswith('@odata'):
                data_details.append(f"{key.capitalize()}: {value}")
        final_data = "\n".join(data_details)
        formatted_data.append(final_data + "\n\n---\n")

    return "\n".join(formatted_data).rstrip("\n\n---\n")



from langchain.pydantic_v1 import BaseModel, Field
from langchain.agents import tool
from dotenv import load_dotenv
from langchain_openai import AzureChatOpenAI
from langchain.prompts import ChatPromptTemplate

load_dotenv()


class Request(BaseModel):
    input: str = Field(description="The question asked by the user.")


@tool(args_schema=Request)
def answer_request(input)-> str:
    """
        A tool to answer the user request if it was related to crm data.      
        Args:
            input (str) : The question that needs to be answered.
           

        Returns:
            str: The answer for the user request.
    """
    chat_template = ChatPromptTemplate.from_messages([
            ("system", f"""
                ***PAY ATTENTION TO THE RULES AND EXAMPLES BELOW BETWEEN THE ''' ''', THEY'RE VERY IMPORTANT.***
                    '''
                    -  You are a CRM specialist, for microsoft dynamics CRM.
            
                    - You are designed to answer user requests if it was crm related.
             

                    '''
           """),
            ("human",input)
        ])

    llm = AzureChatOpenAI(api_version="2023-07-01-preview", azure_deployment="mystery_game")
    messages = chat_template.format_messages(input=input)
    response = llm.invoke(messages)
    
    print(f"This is the response for request tool {response.content}")
    return response.content


   


from langchain.agents import create_openai_tools_agent,AgentExecutor
from langchain_core.prompts import (
    ChatPromptTemplate,
    MessagesPlaceholder,

)
from langchain.chat_models import AzureChatOpenAI

from langchain.prompts import MessagesPlaceholder, ChatPromptTemplate
from dotenv import load_dotenv
from langchain_openai import AzureChatOpenAI
from datetime import datetime
from .syntax import fetchXML_syntax
from .answerRequests import answer_request
load_dotenv()
current_datetime = datetime.now()
current_year = current_datetime.year
current_month = current_datetime.month
current_day = current_datetime.day
current_time = current_datetime.strftime("%H:%M:%S")

date = f"{current_year} / {current_month} / {current_day} - {current_time}"
def create_agent(input):
    
    print(f"This is the agent input: {input}")
    agent_content = f"""
    
    You are an AI-powered chat system, designed to answer questions asked by microsoft dynamics CRM user.
    The user will ask question related to their data and youl shall answerr those questions.
    
    Objective:
    - To understand whether the question needs to be formmated to fetchXML format or not.
    - To format the user request if it was related to CRM data or answer their request.
    - To return the formatted data.
    - To answer user requests if related to crm.
    - refer to the chat history to make sure if the question asked is in refernece to an already answered question

    Task:
    - Answer ONLY related questions to Microsoft Dynamics CRM.
    - You have two tools, you should know exactly which one you shall use.
    - Use the tool when needed.
    - Make the conversation human and interactive as possible,avoid using words like As anAI
    - Answer user requests if related to crm.
    - Appologize on answering if not related to crm.
    - refer to the chat history to make sure if the question asked is in refernece to an already answered question
    - ONLY RETURN THE RETURNED VALUE FROM THE TOOLS,DON'T ADD ANY EXTRA TEXT.

    ONLY USE THE GET_SYNTAX TOOL IF words like these show/display/give/find/view/want/retrieve etc. were mentioned
        -Use this tool if the question includes specific requests to view or manipulate data.
        - ONLY RETURN THE RETURNED VALUE FROM THE TOOL,DON'T ADD ANY EXTRA TEXT.
        -Function Parameters:
            input (str) : The question inputted by the user.
        -Output:
        -THE TOOL WILL RETURN THE FORMATTED DATA - IF THE USER REQUESTS TO VIEW THEIR DATA-.


    ONLY USE THIS ANSWER_REQUEST TOOL IF suser input wasn't data retrieving related.
        -Function Parameters:
            input (str) : The question inputted by the user.
        -Output:
        -THE TOOL WILL RETURN THE SUITABLE RESPONSE.

    Arguments:
         input (str) : the user intent.

    Response Crafting Guidelines:
        - The response must be formatted specificlly as it will be used in the url to access crm data.
        - if you figured out the path and query data, only print the path and query data. Do NOT print anything else.
        - Do not add any extra text, only the values that will go into the url.
        - The response must be in the format of the url that will be used to access the crm data. 
        - If the user query wasn't about retrieveing a certain data, send a suitable response.


    You're in time : {date}
   

    """
    prompt = ChatPromptTemplate.from_messages([
        ("system", agent_content),
        ("user", "{input}"),
        MessagesPlaceholder(variable_name="agent_scratchpad"),
    ])

    
    tools = [fetchXML_syntax,answer_request]  

    model = AzureChatOpenAI(api_version="2023-07-01-preview", azure_deployment="mystery_game")

    agent = create_openai_tools_agent(model, tools, prompt)

    chatbot_instance = AgentExecutor(agent=agent, tools=tools)

    return chatbot_instance



from langchain.pydantic_v1 import BaseModel, Field
from langchain.agents import tool
from dotenv import load_dotenv
from langchain_openai import AzureChatOpenAI
from langchain.prompts import ChatPromptTemplate
load_dotenv()
from datetime import datetime
current_datetime = datetime.now()
current_year = current_datetime.year
current_month = current_datetime.month
current_day = current_datetime.day
current_time = current_datetime.strftime("%H:%M:%S")

class ChatBot(BaseModel):
    input: str = Field(description="The question asked by the user.")
   

@tool(args_schema=ChatBot)
def fetchXML_syntax(input)-> str:
    """
        A tool to format the user request if it was related to crm data.      
        Args:
            input (str) : The question that needs to be formatted.
            history (List) : The chat history.

        Returns:
            str: Formatted data.
    """
    chat_template = ChatPromptTemplate.from_messages([
            ("system", f"""
                ***PAY ATTENTION TO THE RULES AND EXAMPLES BELOW BETWEEN THE ''' ''', THEY'RE VERY IMPORTANT.***
                    '''
                    -  You are a CRM specialist, your role is to analyze user input, and transform it to fetchXML format, that Microsoft Dynamics CRM would understand
            
                    - Do not generate any extra text, only generate the XML format 
            
                    - The format should look something like this:
                        --Example : 
                            - User: "List all accounts."
                              /accounts              
                            - User: "I want to view/check/retrieve/see/look for/ ask about data from account[Entity] where name is equal to MS Account info"
                              /accounts?$select=name$filter=name eq 'MS Account info'
                            - User: "I want to see all contacts."
                              /contacts?$select=fullname,contactid,emailaddress1
                            -User:"Show me all open oppurtunities this quarter"
                              /opportunities?$filter=statecode eq 0 and estimatedclosedate ge 2024-01-01 and estimatedclosedate le 2024-12-31
                            -User:"Show me my meetings for the year 2024 month feb"
                              /appointments?$filter=scheduledstart ge 2024-02-01T00:00:00Z and scheduledstart le 2024-02-28T23:59:59Z and statuscode eq 1
                            - User : "find the full names of all leads that were created on this month"
                                /leads?$select=fullname,createdon&$filter=createdon ge 2024-02-01T00:00:00Z and createdon le 2024-02-28T23:59:59Z


                     - DO NOT GENERATE ANY EXTRA TEXT,  ALL I WANT IS THE FORMATTED DATA. NOT EVEN TO CLARIFY THE USER INTENT, JUST THE FORMATTED DATA.
                     - DO NOT CLARIFY ONLY RETURN THE SINGLE VALUE EXAMPLE: accounts
                     - Just give me the response itself, without using words like: system, human, etc.
                     - If the contact isn't found in the data, respond with "No record found for this user.",For example if the user entered "Give me the account details 
                       for Aseel Alzubaidi",and the user is not found generate No record found for this user.

                     - refer to the chat history to make sure if the question asked is in refernece to an already answered question
             
                     - You're in time : {current_time,current_year,current_month,current_day}

                    '''
    """),
            ("human",input)
        ])

    llm = AzureChatOpenAI(api_version="2023-07-01-preview", azure_deployment="mystery_game")
    messages = chat_template.format_messages(input=input)
    response = llm.invoke(messages)
    return response.content


    

    

   


from langchain_openai import AzureChatOpenAI
from langchain.prompts import ChatPromptTemplate
from dotenv import load_dotenv
load_dotenv()


llm = AzureChatOpenAI(api_version="2023-07-01-preview", azure_deployment="mystery_game")


def handle_invalid_request(intent):
        chat_template = ChatPromptTemplate.from_messages([
            ("system", f"""
                ***PAY ATTENTION TO THE RULES AND EXAMPLES BELOW BETWEEN THE ''' ''', THEY'RE VERY IMPORTANT.***
                    '''
                    -  You are a CRM specialist, for microsoft dynamics CRM.
            
                    - You are designed to inform the user that the data theyre looking for isn't found.
             
                   


                    '''
    """),




            ("human",intent)
        ])




    
        messages = chat_template.format_messages(intent=intent)
        response = llm.invoke(messages)
        
        
        return response.content


from .agent import create_agent
from .get_data import get_data
from .formatData import format_data
from .handle_user_input import handle_invalid_request

def get_response(input : str):
        agent = create_agent(input)
        content = agent.invoke({"input": input})
        if content["output"].startswith("/"):
            result = get_data(content["output"])
            if not result:
                content = handle_invalid_request(input)
                return content
            else:
                if isinstance(result, dict):
                    format = format_data(result)
                    return format
        else:
            return content["output"]
    



from setuptools import setup, find_packages

setup(
    name='crmchatbot',  
    version='0.1.4',
    author='AseelWajieh',
    author_email='aseel.wajieh@gmail.com',
    description='A package for a crm chatbot that can answer user requests if it was related to crm data, and fetch some data based on the user request.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    install_requires=[
        'aiohttp==3.9.3',
        'aiosignal==1.3.1',
        'annotated-types==0.6.0',
        'anyio==4.3.0',
        'attrs==23.2.0',
        'certifi==2024.2.2',
        'charset-normalizer==3.3.2',
        'dataclasses-json==0.6.4',
        'distro==1.9.0',
        'fastapi==0.110.0',
        'frozenlist==1.4.1',
        'h11==0.14.0',
        'httpcore==1.0.4',
        'httpx==0.27.0',
        'idna==3.6',
        'jsonpatch==1.33',
        'jsonpointer==2.4',
        'langchain==0.1.9',
        'langchain-community==0.0.24',
        'langchain-core==0.1.27',
        'langchain-openai==0.0.8',
        'langsmith==0.1.10',
        'marshmallow==3.21.0',
        'multidict==6.0.5',
        'mypy-extensions==1.0.0',
        'numpy==1.26.4',
        'openai==1.12.0',
        'orjson==3.9.15',
        'packaging==23.2',
        'pydantic==2.6.3',
        'pydantic_core==2.16.3',
        'python-dotenv==1.0.1',
        'PyYAML==6.0.1',
        'regex==2023.12.25',
        'requests==2.31.0',
        'sniffio==1.3.1',
        'SQLAlchemy==2.0.27',
        'starlette==0.36.3',
        'tenacity==8.2.3',
        'tiktoken==0.6.0',
        'tqdm==4.66.2',
        'typing-inspect==0.9.0',
        'typing_extensions==4.10.0',
        'urllib3==2.2.1',
        'yarl==1.9.4'],
    url='https://github.com/aseelwajieh/CRM-package',
)

